@@ -1,5 +1,5 @@
 /* mechinfo.c --- Definition of LOGIN mechanism.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002-2019 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -15,13 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with GNU SASL Library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
-#if HAVE_CONFIG_H
-# include "config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
 /* Get specification. */
@@ -48,7 +48,8 @@ Gsasl_mechanism gsasl_login_mechanism = {
    NULL,
 #endif
    NULL,
-   NULL},
+   NULL}
+  ,
   {
    NULL,
    NULL,
