@@ -1,5 +1,5 @@
 /* xstep.c --- Perform one SASL authentication step.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002-2019 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -13,9 +13,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * License along with GNU SASL Library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License License along with GNU SASL Library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -53,9 +54,6 @@ gsasl_step (Gsasl_session * sctx,
 {
   Gsasl_step_function step;
 
-  if (sctx == NULL)
-    return GSASL_INVALID_HANDLE;
-
   if (sctx->clientp)
     step = sctx->mech->client.step;
   else
@@ -101,15 +99,13 @@ gsasl_step64 (Gsasl_session * sctx, const char *b64input, char **b64output)
 
   res = gsasl_step (sctx, input, input_len, &output, &output_len);
 
-  if (input != NULL)
-    free (input);
+  free (input);
 
   if (res == GSASL_OK || res == GSASL_NEEDS_MORE)
     {
       int tmpres = gsasl_base64_to (output, output_len, b64output, NULL);
 
-      if (output != NULL)
-	free (output);
+      free (output);
 
       if (tmpres != GSASL_OK)
 	return tmpres;
